/*
 * Header file for KSI Labs, LLC "CENTIPEDE" interface board.
 * Hardware definitions, bit fields etc. Supposed to be portable
 * between at least Linux and Windows.
 *
 * Author: Sergey Kubushyn <ksi@koi8.net>
 *
 * Copyright (C) 2010 Sergey Kubushyn, KSI Labs, LLC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef _CENTIPEDE_H_
#define _CENTIPEDE_H_

#include <linux/ioctl.h>

/* Register offsets */

/* Output port */
#define CP_DOUT_OFF		0x00
#define CP_DOUT_SET_OFF		0x04
#define CP_DOUT_CLR_OFF		0x08

/* Input port */
#define CP_DIN_OFF		0x0c
#define CP_DEBOUNCE_OFF		0x10
#define CP_IRQMASK_OFF		0x14
#define CP_IRQSENSE_OFF		0x18
#define CP_IRQEDGE_OFF		0x1c
#define CP_IRQBOTH_OFF		0x20
#define CP_IRQSTAT_RAW_OFF	0x24
#define CP_IRQSTAT_OFF		0x28

/* DACs */
#define CP_DAC_A_OFF		0x2c
#define CP_DAC_B_OFF		0x30

/* ADCs */
#define CP_ADC_0_OFF		0x34
#define CP_ADC_1_OFF		0x38
#define CP_ADC_2_OFF		0x3c
#define CP_ADC_3_OFF		0x40

/* Second set of IRQs for ADC/DAC/encoders/etc. */
#define CP_IRQSTAT2_RAW_OFF	0x44
#define CP_IRQMASK2_OFF		0x48
#define CP_IRQSTAT2_OFF		0x4c

/* This are stubs for extension header lines */
#define CP_EX_0_31_OFF		0x50
#define CP_EX_32_41_OFF		0x54
#define CP_EX_0_31_DIR_OFF	0x58
#define CP_EX_32_41_DIR_OFF	0x5c


/* Bits */
#define CP_IRQ_LEVEL	1
#define CP_IRQ_EDGE	0
#define CP_IRQ_RISE	1
#define CP_IRQ_FALL	0
#define CP_IRQ_HIGH	1
#define CP_IRQ_LOW	0
#define EX_DIR_IN	0
#define EX_DIR_OUT	1
#define CP_IRQ_DAC_A	(1 << 0)
#define CP_IRQ_DAC_B	(1 << 1)
#define CP_IRQ_ADC_0	(1 << 2)
#define CP_IRQ_ADC_1	(1 << 3)
#define CP_IRQ_ADC_2	(1 << 4)
#define CP_IRQ_ADC_3	(1 << 5)
#define CP_IRQ_BRKLOST	(1 << 6)
#define CP_BRK_STATUS	(1 << 31)

struct centipede_guts {
	uint32_t	dout_raw;
	uint32_t	dout_set;
	uint32_t	dout_clr;
	uint32_t	din;
	uint32_t	debounce;
	uint32_t	irq_mask;
	uint32_t	irq_sense;
	uint32_t	irq_edge;
	uint32_t	irq_both;
	uint32_t	irq_raw;
	uint32_t	irq_stat;
	uint32_t	dac_a;
	uint32_t	dac_b;
	uint32_t	adc_0;
	uint32_t	adc_1;
	uint32_t	adc_2;
	uint32_t	adc_3;
	uint32_t	irq_raw2;
	uint32_t	irq_mask2;
	uint32_t	irq_stat2;
	uint32_t	ex_data_low;
	uint32_t	ex_data_hi;
	uint32_t	ex_dir_low;
	uint32_t	ex_dir_hi;
};

/* IRQ reason bits (read from centipedeX device upon SIGIO) */
#define IRQ_DIN		(1 << 0)
#define IRQ_DAC		(1 << 1)
#define IRQ_ADC		(1 << 2)
#define IRQ_BRKLOST	(1 << 3)

#define	CP_IOC_MAGIC	0xdf

#define	CP_IOCHOOKIRQ	_IO(CP_IOC_MAGIC, 0)
#define CP_IOCDROPIRQ	_IO(CP_IOC_MAGIC, 1)

#define CP_IOC_MAXNR	1

#endif
